from argparse import ArgumentParser, Namespace
from .core import build_output, infer_params
from .utils import find_target, load_config
from pathlib import Path
from typing import Dict


def build_parser():
    parser = ArgumentParser()
    parser.add_argument('-t', '--target', type=str, default='common', help='tfvars search target')
    parser.add_argument('-k', '--key', type=str, help='variable to read')
    parser.add_argument('-c', '--component', choices=['bucket', 'object', 'table', 'env'])
    parser.add_argument('-x', '--extension', type=str, default='tfvars')
    parser.add_argument('--bucket-name', type=str, default='terraformstate')
    parser.add_argument('--table-name', type=str, default='terraformlock')
    parser.add_argument('--prefix', type=str, default='{app_name}-')
    parser.add_argument('--suffix', type=str, default='-{app_env}-{company}')
    parser.add_argument('--state-name', type=str, default='terraform.tfstate')
    parser.add_argument('--env-var', type=str, default='app_env')
    parser.add_argument('--env-pos', type=int, default=0)
    return parser


def main():
    parser: ArgumentParser = build_parser()
    args: Namespace = parser.parse_args()
    name: str = f'{args.target}.{args.extension}'
    target: Path = find_target(name)
    if not target:
        return
    config_data: Dict[str, str] = load_config(target)
    infer_data: Dict[str, str] = infer_params(project_dir=target.parent, app_env_var=args.env_var, app_env_pos=args.env_pos)
    data: Dict[str, str] = {**config_data, **infer_data}
    output: str = build_output(data, key=args.key, component=args.component, prefix=args.prefix, suffix=args.suffix,
                               bucket_name=args.bucket_name, table_name=args.table_name, state_name=args.state_name,
                               app_env_var=args.env_var)
    if output:
        print(output, end='')



from typing import Dict, List, Optional, Union
from pathlib import Path, PurePosixPath

__all__ = ['infer_params', 'build_output', 'setup_project']


def infer_params(project_dir: Path, work_dir: Path = None, construct_var: str = 'construct',
                 app_env_var: str = 'app_env', app_env_pos: int = 0) -> Dict[str, str]:
    work_dir: Path = work_dir or Path().absolute()
    relative_path: Path = work_dir.relative_to(project_dir)
    construct_parts = list(relative_path.parts)
    return {app_env_var: construct_parts.pop(app_env_pos), construct_var: str(Path(*construct_parts))}


def build_output(data: Dict[str, str], key: str = None, component: str = None,
                 prefix: str = '{app_name}-', suffix: str = '-{app_env}-{company}',
                 bucket_name: str = 'terraformstate', table_name: str = 'terraformlock',
                 state_name: str = 'terraform.tfstate', app_env_var: str = 'app_env') -> str:
    if key:
        return data.get(key)

    if component == 'env':
        return data[app_env_var]

    if component == 'bucket':
        return prefix.format(**data) + bucket_name + suffix.format(**data)

    if component == 'table':
        return prefix.format(**data) + table_name + suffix.format(**data)

    if component == 'object':
        state_path = Path(data['construct']) / state_name
        state_path = PurePosixPath(state_path)
        return str(state_path)


def setup_project(path: Optional[Union[Path, str]], create: bool = True,
                  constructs: List[str] = None, envs: List[str] = None,
                  common_name: str = 'common', common_ext: str = 'tfvars',
                  terragrunt_name: str = 'terragrunt', terragrunt_ext: str = 'hcl'):

    live_dir: Union[Path, str] = path or Path()
    live_dir: Path = Path(live_dir)
    live_dir.mkdir(exist_ok=create)

    envs = envs or ['sbx', 'dev', 'qa', 'uat', 'prd']
    constructs = constructs or ['params', 'network', 'iam', 'app']

    for env in envs:
        for construct in constructs:
            terragrunt_hcl = live_dir / env / construct / f'{terragrunt_name}.{terragrunt_ext}'
            terragrunt_hcl.parent.mkdir(exist_ok=True, parents=True)
            terragrunt_hcl.touch(exist_ok=True)

    terragrunt_hcl = live_dir / f'{terragrunt_name}.{terragrunt_ext}'
    common_tfvars = live_dir / f'{common_name}.{common_ext}'

    terragrunt_hcl.touch(exist_ok=True)
    common_tfvars.touch(exist_ok=True)



from setuptools import setup
from typing import Union
from pathlib import Path
import subprocess
import re


def read_file(path: Union[str, Path]) -> str:
    with Path(path).open('r') as f:
        return f.read().strip()


def infer_version() -> str:
    process = subprocess.run(['git', 'describe'], stdout=subprocess.PIPE)
    output = process.stdout.decode('utf-8').strip()
    version = re.sub('^v', '', output)
    return version


setup(name='terraform-utils',
      version=infer_version(),
      url='https://github.com/rkhullar/terraform-utils',
      author='Rajan Khullar',
      author_email='rkhullar03@gmail.com',
      long_description=read_file('readme.md'),
      long_description_content_type='text/markdown',
      keywords='terraform terragrunt',
      license='MIT',
      packages=['terraform_utils'],
      install_requires=[],
      include_package_data=True,
      zip_safe=False,
      test_suite='nose.collector',
      tests_require=['nose', 'parameterized'],
      entry_points={'console_scripts': ['tf-util=terraform_utils.command_line:main']}
      )
